function Tile(){
	this.timesExplored = 0;
	this.light = 0;
	this.lightVisits = {};
	
	this.effects = [];
	this.explored = false;
	this.playerVisible = false;
	
	if(Math.random()<.5){
		this.preferredMonster = "human";
	}else{
		this.preferredMonster = "vampire";
	}
	
	this.description = "";

	this.isHallway = false;
	
	this.tutorial = false;
	
	this.preferDoor = false;
	this.monster = null;
	
	this.penetrated = false;
	
	this.torch = 0;
	
	this.item = null;
	
	this.id = Math.random().toString();
}



Tile.prototype.getDescription = function(){
	return this.description;
}

Tile.prototype.getName = function(){
	return this.name;
}

Tile.prototype.draw= function(i,j){
	if(game.fullVisibilty){
		this.light = Math.max(1,this.light);
		this.playerVisible = true;
	}

	if(this.light>=player.visibilityThreshold && this.playerVisible){
		ctx.globalAlpha = Math.min(1,this.light*player.lightingMultiplier);
		if(player.isVampire()){
			ctx.globalAlpha = Math.max(.2,ctx.globalAlpha);
		}
		//ctx.globalAlpha = 1.5;
	
		ssl = ssLocation(this.getSsIndex());
		ctx.drawImage(ss,ssl[0],ssl[1],16,16,i*tileSize,j*tileSize,tileSize,tileSize);
		
		ctx.globalAlpha = 1;
		
		if(this.torch>0){
			ssl = ssLocation(4+this.torch);
			ctx.drawImage(ss,ssl[0],ssl[1],16,16,i*tileSize,j*tileSize,tileSize,tileSize);
		}
		
		
		if(this.item!=null){
			this.item.draw(i,j);
		}
		
		
		
		for(var k=0;k<this.effects.length;k++){
			ssl = ssLocation(this.effects[k]);
			ctx.drawImage(ss,ssl[0],ssl[1],16,16,i*tileSize,j*tileSize,tileSize,tileSize);
		}
		
		
		if(this.monster!==null){
			this.monster.draw(i,j);
		}
	}else{
		if(this.monster!==null && this.monster.type!="obstacle" && player.detect){
			var ssl = ssLocation(68);
			ctx.drawImage(ss,ssl[0],ssl[1],16,16,i*tileSize,j*tileSize,tileSize,tileSize);
		}
	}
}

Tile.prototype.stepOn = function(monster){
	if(!monster){
		if(this.item!=null){
			if(this.item.pickUp()){
				game.log("You pick up the "+this.item.getName()+".");
				this.item = null;
			}
		}
	}
}

Tile.prototype.getSsIndex = function(){
	return this.ssIndex;
}

//assumes same level
Tile.prototype.equals = function(tile){
	return (this.i==tile.i && this.j==tile.j);
}

Tile.prototype.isPassable = function(){
	return this.passable;
}

Tile.prototype.generationExplore = function(){
	return this.timesExplored++;
}

Tile.prototype.getTimesExplored = function(){
	return this.timesExplored;
}

Tile.prototype.getAdjacentNeighbors = function(shuffle){
	var tiles = [
		this.level.getTile(this.i+1,this.j),
		this.level.getTile(this.i-1,this.j),
		this.level.getTile(this.i,this.j+1),
		this.level.getTile(this.i,this.j-1)
	];
	if(shuffle){
		return game.shuffle(tiles);
	}else{
		return tiles;
	}
}

Tile.prototype.getPassableNeighbors = function(){
	var passables = [];
	var neighbors = this.getAdjacentNeighbors();
	for(var i=0;i<neighbors.length;i++){
		if(neighbors[i].isPassable()){
			passables.push(neighbors[i]);
		}
	}
	return passables;
}

Tile.prototype.isAdjacent = function(tile){
	var neighbors = this.getAdjacentNeighbors();
	for(var i=0;i<neighbors.length;i++){
		if(neighbors[i].equals(tile)){
			return true;
		}
	}
	return false;
}

Tile.prototype.isNeighbor = function(tile){
	if(this.i>=tile.i-1
	&& this.i<=tile.i+1
	&& this.j>=tile.j-1
	&& this.j<=tile.j+1){
		return true;
	}else{
		return false;
	}
}

Tile.prototype.isNearby = function(tile){
	if(this.i>=tile.i-playerOffset
	&& this.i<=tile.i+playerOffset
	&& this.j>=tile.j-playerOffset
	&& this.j<=tile.j+playerOffset){
		return true;
	}else{
		return false;
	}
}

Tile.prototype.getAllNeighbors = function(){
	return [
		this.level.getTile(this.i+1,this.j),
		this.level.getTile(this.i-1,this.j),
		this.level.getTile(this.i,this.j+1),
		this.level.getTile(this.i,this.j-1),
		this.level.getTile(this.i+1,this.j+1),
		this.level.getTile(this.i-1,this.j-1),
		this.level.getTile(this.i-1,this.j+1),
		this.level.getTile(this.i+1,this.j-1),
	];
}





function StoneWall(i,j,level){
	this.i=i;
	this.j=j;
	this.level=level;

	Tile.call(this);
	
	this.ssIndex = 1;
	this.type = "wall";
	this.wallType = "stone";
	this.opaque = true;
	
	this.passable = false;
	this.name = "Stone Wall";

	if(this.level.index == map.MAUSOLEUM_LEVEL && Math.random()<.2){
		this.skulls = true;
	}
}

StoneWall.prototype = new Tile();
StoneWall.prototype.constructor = Tile;

StoneWall.prototype.getSsIndex = function(){
	if(this.penetrated){
		return 34;
	}else{
		if(this.skulls){
			return 28;
		}else{
			return 1;
		}
	}
}

StoneWall.prototype.getDescription = function(){
	if(this.penetrated){
		return "A half-demolished stone wall.";
	}else{
		return "Your run of the mill stone wall.";
	}
}



function Entry(i,j,level){
	this.i=i;
	this.j=j;
	this.level=level;

	Tile.call(this);
	
	this.ssIndex = 55;
	this.type = "wall";
	this.wallType = "entry";
	this.opaque = true;
	
	this.passable = false;
	this.name = "Entrance";
	
	this.description ="The entrance to the Golden Krone Hotel. Why check out now?";
}

Entry.prototype = new Tile();
Entry.prototype.constructor = Tile;





function WoodenWall(i,j,level){
	this.i=i;
	this.j=j;
	this.level=level;

	Tile.call(this);
	
	this.bookshelf = false;
	this.ssIndex = 16;
	if(Math.random()<.005){
		this.painting = true;
	}else{
		this.painting = false;
		if((this.level.index == map.LIBRARY_LEVEL && Math.random()<.85) || Math.random()<.005){
			this.bookshelf = true;
		}
	}

	this.type = "wall";
	this.wallType = "wooden";
	this.opaque = true;
	
	this.passable = false;
	this.name = "Wooden Wall";
}

WoodenWall.prototype = new Tile();
WoodenWall.prototype.constructor = Tile;

WoodenWall.prototype.getSsIndex = function(){
	if(this.penetrated){
		if(this.bookshelf){
			return 29;
		}else{
			return 35;
		}
	}else{
		if(this.painting){
			return 42;
		}else if(this.bookshelf){
			return 27;
		}else{
			return 16;
		}
	}
}

WoodenWall.prototype.getDescription = function(){
	if(this.penetrated){
		return "A half-demolished wooden wall.";
	}else{
		return "Your standard wooden wall.";
	}
}



function StainedGlass(i,j,level){
	this.i=i;
	this.j=j;
	this.level=level;

	Tile.call(this);
	
	this.ssIndex = 18;
	this.type = "wall";
	this.wallType = "stained";
	this.opaque = true;
	
	this.passable = false;
	this.name = "Stained Glass";
}

StainedGlass.prototype = new Tile();
StainedGlass.prototype.constructor = Tile;


StainedGlass.prototype.getDescription = function(){
	if(this.penetrated){
		return "A once beautiful stained glass window. Colorful shards are scattered everywhere. It appears to have been smashed.";
	}else{
		return "An ornately designed stained glass window. It allows surprisingly little light through.";
	}
}

StainedGlass.prototype.getSsIndex = function(){
	if(this.penetrated){
		return 19;
	}else{
		return 18;
	}
}



function Glass(i,j,level){
	this.i=i;
	this.j=j;
	this.level=level;

	Tile.call(this);
	
	this.ssIndex = 18;
	this.type = "wall";
	this.wallType = "glass";
	this.opaque = true;
	
	this.passable = false;
	this.name = "Glass";
}

Glass.prototype = new Tile();
Glass.prototype.constructor = Tile;


Glass.prototype.getDescription = function(){
	if(this.penetrated){
		return "A smashed glass window.";
	}else{
		return "A simple glass window. Unlike stained glass, it lets plenty of light through.";
	}
}

Glass.prototype.getSsIndex = function(){
	if(this.penetrated){
		return 26;
	}else{
		return 25;
	}
}



function Floor(i,j,level){
	this.i=i;
	this.j=j;
	this.level=level;
	
	Tile.call(this);
	
	this.ssIndex = 121+16*this.level.floorSubtype;
	this.type = "floor";
	this.opaque = false;
	
	this.passable = true;
	this.name = "Floor";
	
	this.description = "You wonder when the housekeepers last attended to this area.";
}

Floor.prototype = new Tile();
Floor.prototype.constructor = Tile;


function Staircase(i,j,level){
	this.i=i;
	this.j=j;
	this.level=level;
	
	Tile.call(this);
	
	this.ssIndex = 0;
	this.type = "staircase";
	this.opaque = false;

	this.branch = false;
	
	this.direction = "down"
	
	this.passable = true;

	this.name = "Staircase";
}

Staircase.prototype = new Tile();
Staircase.prototype.constructor = Tile;

Staircase.prototype.getSsIndex = function(){
	if(this.branch){
		if(this.direction=="down"){
			return 124+16*this.level.floorSubtype;
		}else{
			return 126+16*this.level.floorSubtype;
		}
	}else{
		if(this.direction=="down"){
			return 122+16*this.level.floorSubtype;
		}else{
			if(this.level.index == map.FANE_LEVEL-1 && game.faneLocked){
				return 125+16*this.level.floorSubtype;
			}else{
				return 123+16*this.level.floorSubtype;
			}
		}
	}
}

Staircase.prototype.getName = function(){
	var name;
	if(this.branch && this.direction=="down"){
		var branchIndex = this.level.index+(1+map.numBranches);
		name = "Staircase to The "+map.branchNames[branchIndex];
	}else{
		name = "Staircase";
	}
	return name;
}

Staircase.prototype.getDescription = function(){
	if(this.branch){
		if(this.direction=="down"){
			var branchIndex = map.levelIndex+(1+map.numBranches);
			var downText = "A long, winding staircase to the "+ map.branchNames[branchIndex]+".<br><br>";
			downText += map.branchDescriptions[branchIndex];
			return downText;
		}else{
			return "A long, winding staircase back to the hotel proper.";
		}
	}else{
		if(this.direction=="down"){
			return "A staircase leading down a floor.";
		}else{
			if(this.level.index == map.FANE_LEVEL-1 && game.faneLocked){
				return "An inactive magical staircase. In its current state, you won't be able to climb up. Something in the hotel must control this staircase...";
			}else{
				return "A staircase leading up a floor.";
			}
		}
	}
}





function Mantrap(i,j,level){
	this.i=i;
	this.j=j;
	this.level=level;
	
	Tile.call(this);

	this.type = "mantrap";
	this.opaque = false;

	this.trapState = 0;
	
	this.passable = true;

	this.name = "Venus Mantrap";

	this.description = "Like its smaller cousin, the Venus Mantrap has a complex mechanism for detecting prey. Unlike the flytrap, however, it doesn't waste time on digesting its prey whole. Instead the Mantrap prefers to simply eviscerate large animals and feast on their entrails."+
	"<br><br>&ldquo;Astonishing, isn't it, gentlemen? That plant is the vampire of the vegetable kingdom.&rdquo; - <em>Nosferatu</em>";
}

Mantrap.prototype = new Tile();
Mantrap.prototype.constructor = Tile;

Mantrap.prototype.stepOn = function(monster){
	if(this.trapState == 0){
		this.trapState = 1;
		this.trapTime = game.time;
	}else if(this.trapState == 1 && (game.time-this.trapTime) > 30){
		this.trapState = 2;
		game.log("The movement triggers the Venus Mantrap and its lobes snap togeher violently.")
		if(monster){
			monster.hit(60, null, "The Venus Mantrap bites the ");
			game.playSound("hit3");
		}else{
			player.hit(60,"The Venus Mantrap");
		}
	}
}

Mantrap.prototype.getSsIndex = function(){
	if(this.trapState==0){
		return 170;
	}else if(this.trapState==1){
		return 171;
	}else{
		return 172;
	}
}


function Door(i,j,level){
	this.i=i;
	this.j=j;
	this.level=level;
	
	Tile.call(this);
	
	this.ssIndex = 17;
	this.type = "door";
	this.opaque = true;
	
	this.open = false;
	
	this.passable = false;
	this.name = "Door";
	
	this.description = "A closed door. Could be an open door one day.";
}

Door.prototype = new Tile();
Door.prototype.constructor = Tile;

Door.prototype.stepOn = function(monster){
	if(!this.monster){
		this.openDoor();
	}
}

Door.prototype.openDoor = function(){
	this.level.setTile(this.i,this.j,new Floor(this.i,this.j,this.level),false);
}